red = 'Red'
blue = 'Blue'
green = 'Green'
spring_green = 'SpringGreen'
hot_pink = 'HotPink'
blue_violet = 'BlueViolet'
cadet_blue = 'CadetBlue'
chocolate = 'Chocolate'
coral = 'Coral'
dodger_blue = 'DodgerBlue'
firebrick = 'Firebrick'
golden_rod = 'GoldenRod'
orange_red = 'OrangeRed'
yellow_green = 'YellowGreen'
sea_green = 'SeaGreen'